var Block = function()
{
	var block = {};
	var debug = 0;
	
	var workList;

	// Page initialization
	block.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', block.validateStep1);
		Events.addEventHandler('embedDate', 'blur', Wizard.validateDate);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', block.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', block.validateStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', block.submit);
		
		Reggie.loadProtocols('HISTOLOGY_PROTOCOL', 'histologyProtocol');

		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetHistologyWorkLists';
		Wizard.showLoadingAnimation('Loading histology work lists...');
		Wizard.asyncJsonRequest(url, block.histologyListsLoaded);
	}
	
	block.histologyListsLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var workLists = response.workLists;
		
		if (workLists.length > 0)
		{
			for (var i=0; i < workLists.length; i++)
			{
				var list = workLists[i];
				var name = list.name + ' (' + list.size + ')';
				var option = new Option(name, list.id);
				frm.workList[frm.workList.length] = option;
			}
			
			Wizard.setInputStatus('workList', 'valid');
		}
		else
		{
			Wizard.setFatalError('No Histology work lists available for processing.');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gonext');
		frm.embedDate.focus();
	}

	block.validateStep1 = function(event)
	{
		var valid = Wizard.isValid('embedDate');
		if (!valid) event.preventDefault();
	}

	
	block.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetHistologyWorkListInfo&workListId='+frm.workList.value;		
		Wizard.showLoadingAnimation('Loading histology work list information...');
		Wizard.asyncJsonRequest(url, block.workListInfoLoaded);
	}
	
	block.workListInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		workList = response.workList;
		var lastBlockBox = Data.get('page-data', 'last-block-box', '');

		var html = '<table id="blockTable">';
		html += '<thead>';
		// First header row
		html += '<tr>';
		html += '<th style="width: 8em;">Paraffin block</th>';
		html += '<th style="width: 8em;">Storage box</th>';
		html += '<th style="width: 25em;">Comment</th>';
		html += '<th style="width: 20px;"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		for (var blockNo = 0; blockNo < workList.blocks.length; blockNo++)
		{
			var pb = workList.blocks[blockNo];

			html += '<tr>';
			html += '<td>' + Strings.encodeTags(pb.name) + '</td>';
			html += '<td><input type="text" name="blockBox.'+blockNo+'" id="blockBox.'+blockNo+'" value="'+lastBlockBox+'" style="width: 8em;"></td>';
			html += '<td><input type="text" name="blockComment.'+blockNo+'" id="blockComment.'+blockNo+'" style="width: 25em;"></td>';
			html += '<td class="status" id="blockBox.'+blockNo+'.status"></td>';
			html += '</tr>';
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('blockBoxInput').innerHTML = html;
		
		for (var blockNo = 0; blockNo < workList.blocks.length; blockNo++)
		{
			Events.addEventHandler('blockBox.'+blockNo, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'blockBox.'+(blockNo+1) });
			Events.addEventHandler('blockBox.'+blockNo, 'change', block.blockBoxOnChange);
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('gocancel');
		frm['blockBox.0'].focus();
	}

	/**
		Propagate changes to downward blocks.
	*/
	block.blockBoxOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var blockName = event.currentTarget.value;

		var isDownwards = false;
		for (var blockNo = 0; blockNo < workList.blocks.length; blockNo++)
		{
			var bb = frm['blockBox.'+blockNo];
			if (isDownwards)
			{
				bb.value = blockName;
			}
			else if (bb == event.currentTarget)
			{
				isDownwards = true;
			}
		}
	}

	block.validateStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		var valid = true;
		Wizard.setInputStatus('blockBox');
		
		for (var blockNo = 0; blockNo < workList.blocks.length; blockNo++)
		{
			if (frm['blockBox.'+blockNo].value == '')
			{
				Wizard.setInputStatus('blockBox.'+blockNo, 'invalid', 'A storage box must be specified for all blocks.');
				valid = false;
			}
			else
			{
				Wizard.setInputStatus('blockBox.'+blockNo, 'valid');
			}
		}
		if (!valid)
		{
			Wizard.setInputStatus('blockBox', '', 'A storage box must be specified for all blocks.');
			event.preventDefault();
		}
	}

	
	block.submit = function()
	{
		var frm = document.forms['reggie'];
		
		workList.embedDate = frm.embedDate.value;
		workList.protocolId = parseInt(frm.histologyProtocol.value, 10);

		for (var blockNo = 0; blockNo < workList.blocks.length; blockNo++)
		{
			var pb = workList.blocks[blockNo];
			pb.storageBox = frm['blockBox.'+blockNo].value;
			pb.comment = frm['blockComment.'+blockNo].value;
		}

		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=MoveHistologySamplesToParaffinBlocks';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, block.submissionResults, 'POST', JSON.stringify(workList));
		
	}
	
	block.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return block;
}();

Doc.onLoad(Block.initPage);

